@@ -101,7 +101,7 @@ extern "C" {
 #define SET_ENCODING 106
 #define INQ_ENCODING 107
 #define SET_RESAMPLE_METHOD 108
-#define SET_RESIZE_BEHAVIOUR 109
+#define SET_NOMINAL_SIZE 109
 #define INQ_TEXT 110
 
 #define SET_TEXT_SLANT 200
@@ -116,6 +116,7 @@ extern "C" {
 #define INQ_WS_STATE 209
 #define SAMPLE_LOCATOR 210
 #define SET_CLIP_REGION 211
+#define SET_CLIP_SECTOR 212
 
 #define BEGIN_SELECTION 250
 #define END_SELECTION 251
@@ -126,6 +127,9 @@ extern "C" {
 #define GKS_SET_BBOX_CALLBACK 260
 #define GKS_CANCEL_BBOX_CALLBACK 261
 
+#define SET_BACKGROUND 262
+#define CLEAR_BACKGROUND 263
+
 #define ENCODING_LATIN1 300
 #define ENCODING_UTF8 301
 
@@ -170,10 +174,12 @@ typedef struct
   double bwidth;
   int bcoli;
   int clip_tnr;
-  int resize_behaviour;
-  double aspect_ratio;
   int clip_region;
+  double clip_start_angle, clip_end_angle;
+  double nominal_size;
+  double aspect_ratio;
   char *(*callback)(const char *);
+  int in_exit_handler;
   int debug;
 } gks_state_list_t;
 
@@ -312,7 +318,7 @@ DLLEXPORT int gks_dl_read_item(char *dl, gks_state_list_t **gkss,
 void gks_wiss_dispatch(int fctid, int wkid, int segn);
 int gks_debug(void);
 
-#ifndef EMSCRIPTEN
+#ifndef __EMSCRIPTEN__
 
 void gks_drv_mo(int fctid, int dx, int dy, int dimx, int *i_arr, int len_f_arr_1, double *f_arr_1, int len_f_arr_2,
                 double *f_arr_2, int len_c_arr, char *c_arr, void **ptr);
