@@ -914,17 +914,25 @@ static void set_clip_rect(int tnr)
           tnr = gkss->clip_tnr;
         }
       if (p->scoped) pgf_printf(p->stream, "\\end{scope}\n");
-      if (gkss->clip_region == GKS_K_REGION_ELLIPSE)
+      if (gkss->clip_region == GKS_K_REGION_ELLIPSE && tnr != 0)
         {
           double x, y, w, h;
           x = (p->rect[tnr][0][0] + p->rect[tnr][1][0]) / 2;
           y = (p->rect[tnr][0][1] + p->rect[tnr][1][1]) / 2;
           w = p->rect[tnr][1][0] - p->rect[tnr][0][0];
           h = p->rect[tnr][1][1] - p->rect[tnr][0][1];
-          pgf_printf(p->stream,
-                     "\\begin{scope}\n"
-                     "\\clip (%f,%f) ellipse (%f and %f);\n",
-                     x, y, w / 2, h / 2);
+          if (gkss->clip_start_angle > 0 || gkss->clip_end_angle < 360)
+            pgf_printf(p->stream,
+                       "\\begin{scope}\n"
+                       "\\clip (%f,%f) arc (%f:%f:%f and %f) -- (%f,%f);\n",
+                       x + 0.5 * w * cos(-gkss->clip_start_angle * M_PI / 180),
+                       y + 0.5 * h * sin(-gkss->clip_start_angle * M_PI / 180), -gkss->clip_start_angle,
+                       -gkss->clip_end_angle, w / 2, h / 2, x, y);
+          else
+            pgf_printf(p->stream,
+                       "\\begin{scope}\n"
+                       "\\clip (%f,%f) ellipse (%f and %f);\n",
+                       x, y, w / 2, h / 2);
         }
       else
         {
@@ -1486,6 +1494,8 @@ void gks_pgfplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double
       p->height = 500;
       p->width = 500;
       p->nominal_size = 1;
+      if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
+
       p->window[0] = p->window[2] = 0.0;
       p->window[1] = p->window[3] = 1.0;
       p->viewport[0] = p->viewport[2] = 0;
@@ -1679,10 +1689,8 @@ void gks_pgfplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double
 
           p->width = p->viewport[1] * WIDTH / MWIDTH;
           p->height = p->viewport[3] * HEIGHT / MHEIGHT;
-          if (gkss->resize_behaviour == GKS_K_RESIZE)
-            {
-              p->nominal_size = min(p->width, p->height) / 500.0;
-            }
+          p->nominal_size = min(p->width, p->height) / 500.0;
+          if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
 
           set_xform();
           init_norm_xform();
@@ -1690,6 +1698,12 @@ void gks_pgfplugin(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double
         }
       break;
 
+    case 109:
+      /* set nominal size */
+      p->nominal_size = min(p->width, p->height) / 500.0;
+      if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
+      break;
+
     case 203:
       /* set transparency */
       set_transparency(r1[0]);
