@@ -451,14 +451,8 @@ static void set_xform(double *wn, double *vp)
 
   p->width = p->a;
   p->height = p->c;
-  if (gkss->resize_behaviour == GKS_K_RESIZE)
-    {
-      p->nominal_size = MIN(p->width, p->height) / 500.0 * 72 / 600;
-    }
-  else
-    {
-      p->nominal_size = 72.0 / 600;
-    }
+  p->nominal_size = MIN(p->width, p->height) / 500.0 * 72 / 600;
+  if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
 
   p->stroke = 0;
 }
@@ -792,25 +786,48 @@ static void update(void)
     }
 }
 
-static void set_clip(double *clrt)
+static void set_clip_rect(int tnr)
 {
   int i, j;
-  int ix1, ix2, iy1, iy2;
-  double cx1, cy1, cx2, cy2;
+  double *clrt, cx1, cy1, cx2, cy2;
   char buffer[120];
 
+  if (gkss->clip_tnr != 0)
+    tnr = gkss->clip_tnr;
+  else if (gkss->clip == GKS_K_NOCLIP)
+    tnr = 0;
+
+  clrt = gkss->viewport[tnr];
   i = clrt[0] < clrt[1] ? 0 : 1;
   j = clrt[2] < clrt[3] ? 2 : 3;
 
   NDC_to_DC(clrt[i], clrt[j], cx1, cy1);
   NDC_to_DC(clrt[1 - i], clrt[5 - j], cx2, cy2);
 
-  ix1 = ((int)cx1) - 2;
-  iy1 = ((int)cy1) - 2;
-  ix2 = NINT(cx2) + 2;
-  iy2 = NINT(cy2) + 2;
+  if (gkss->clip_region == GKS_K_REGION_ELLIPSE && tnr != 0)
+    {
+      double x, y, rx, ry;
 
-  snprintf(buffer, 120, "np %d %d m %d %d l %d %d l %d %d l cp clip", ix1, iy1, ix1, iy2, ix2, iy2, ix2, iy1);
+      x = 0.5 * (cx1 + cx2);
+      y = 0.5 * (cy1 + cy2);
+      rx = 0.5 * (cx2 - cx1);
+      ry = 0.5 * (cy2 - cy1);
+
+      snprintf(buffer, 120, "np %.2f %.2f m %.2f %.2f l %.2f %.2f %.2f %.2f %.2f %.2f ellipse clip", x, y,
+               x + rx * cos(gkss->clip_start_angle * M_PI / 180), y + ry * sin(gkss->clip_start_angle * M_PI / 180), x,
+               y, rx, ry, gkss->clip_start_angle, gkss->clip_end_angle);
+    }
+  else
+    {
+      int ix1, ix2, iy1, iy2;
+
+      ix1 = ((int)cx1) - 2;
+      iy1 = ((int)cy1) - 2;
+      ix2 = NINT(cx2) + 2;
+      iy2 = NINT(cy2) + 2;
+
+      snprintf(buffer, 120, "np %d %d m %d %d l %d %d l %d %d l cp clip", ix1, iy1, ix1, iy2, ix2, iy2, ix2, iy1);
+    }
   packb(buffer);
 }
 
@@ -1105,6 +1122,17 @@ static void ps_init(int *pages)
       packb("/Encoding exch def dup");
       packb("/FontName exch def currentdict");
       packb("definefont end} def");
+      packb("/ellipse {\
+ /endangle exch def\
+ /startangle exch def\
+ /yrad exch def\
+ /xrad exch def\
+ /y exch def\
+ /x exch def\
+ /savematrix matrix currentmatrix def\
+ x y translate xrad yrad scale 0 0 1 startangle endangle arc\
+ savematrix setmatrix\
+} def");
       packb("end");
 
       packb("%%EndProcSet");
@@ -1146,7 +1174,6 @@ static void ps_init(int *pages)
   set_markersize(-1.0);
   packb("0 ma");
   set_font(-1);
-  set_clip(p->window);
   packb("%%EndPageSetup");
   update();
 }
@@ -1202,15 +1229,19 @@ static void query_color(int index, unsigned char **buf, int wtype)
     }
 }
 
-static void rgb2color(int rgb, unsigned char **buf, int wtype)
+static void rgb2color(unsigned int rgb, unsigned char **buf, int wtype)
 {
-  int r, g, b;
+  int r, g, b, a;
   double grey;
 
   r = (rgb & 0xff);
   g = (rgb & 0xff00) >> 8;
   b = (rgb & 0xff0000) >> 16;
-
+  a = (rgb & 0xff000000) >> 24;
+  if (a == 0)
+    {
+      r = g = b = 0xff;
+    }
   if (wtype % 2)
     {
       grey = 0.3 * r / 255.0 + 0.59 * g / 255.0 + 0.11 * b / 255.0;
@@ -1301,7 +1332,7 @@ static void cell_array(double xmin, double xmax, double ymin, double ymax, int d
 
   packb("gsave");
 
-  set_clip(gkss->viewport[gkss->clip == GKS_K_CLIP ? tnr : 0]);
+  set_clip_rect(gkss->cntnr);
 
   packb("/RawData currentfile /ASCII85Decode filter def");
   packb("/Data RawData << >> /LZWDecode filter def");
@@ -1350,7 +1381,7 @@ static void cell_array(double xmin, double xmax, double ymin, double ymax, int d
           if (!true_color)
             query_color(ci, &bufP, wtype);
           else
-            rgb2color(ci, &bufP, wtype);
+            rgb2color((unsigned int)ci, &bufP, wtype);
         }
     }
   LZWEncodeImage(len, buf);
@@ -1427,7 +1458,7 @@ static void fill_routine(int n, double *px, double *py, int tnr)
 
   packb("gsave");
 
-  set_clip(gkss->viewport[gkss->clip == GKS_K_CLIP ? tnr : 0]);
+  set_clip_rect(gkss->cntnr);
 
   WC_to_NDC(px[0], py[0], tnr, x, y);
   NDC_to_DC(x, y, p->ix, p->iy);
@@ -1989,16 +2020,10 @@ void gks_drv_ps(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r
           if (ltype != GKS_K_LINETYPE_SOLID) set_linetype(ltype, width);
           set_linewidth(width);
           set_color(color, p->wtype);
-          if (gkss->clip_tnr != 0)
-            {
-              packb("gsave");
-              set_clip(gkss->viewport[gkss->clip_tnr]);
-            }
+          packb("gsave");
+          set_clip_rect(gkss->cntnr);
           line_routine(ia[0], r1, r2, ltype, tnr);
-          if (gkss->clip_tnr != 0)
-            {
-              packb("grestore");
-            }
+          packb("grestore");
           if (ltype != GKS_K_LINETYPE_SOLID) set_linetype(GKS_K_LINETYPE_SOLID, 1.0);
           p->empty = 0;
         }
@@ -2024,20 +2049,14 @@ void gks_drv_ps(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r
           set_markerangle(angle);
           color = gkss->asf[5] ? gkss->pmcoli : 1;
           set_foreground(color, p->wtype);
-          if (gkss->clip_tnr != 0)
-            {
-              packb("gsave");
-              set_clip(gkss->viewport[gkss->clip_tnr]);
-            }
+          packb("gsave");
+          set_clip_rect(gkss->cntnr);
           set_linewidth(gkss->bwidth);
           set_bordercolor(p->wtype);
           packb("0 setlinejoin");
           gks_emul_polymarker(ia[0], r1, r2, marker_routine);
           packb("1 setlinejoin");
-          if (gkss->clip_tnr != 0)
-            {
-              packb("grestore");
-            }
+          packb("grestore");
           p->empty = 0;
         }
       break;
@@ -2061,11 +2080,8 @@ void gks_drv_ps(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r
             set_linewidth(1.0);
           color = gkss->asf[9] ? gkss->txcoli : 1;
           set_color(color, p->wtype);
-          if (gkss->clip_tnr != 0)
-            {
-              packb("gsave");
-              set_clip(gkss->viewport[gkss->clip_tnr]);
-            }
+          packb("gsave");
+          set_clip_rect(gkss->cntnr);
           nchars = strlen(chars);
           if (prec == GKS_K_TEXT_PRECISION_STRING)
             {
@@ -2078,10 +2094,7 @@ void gks_drv_ps(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r
             {
               gks_emul_text(r1[0], r2[0], nchars, chars, line_routine, fill_routine);
             }
-          if (gkss->clip_tnr != 0)
-            {
-              packb("grestore");
-            }
+          packb("grestore");
           p->empty = 0;
         }
       break;
@@ -2101,11 +2114,8 @@ void gks_drv_ps(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r
           color = gkss->asf[12] ? gkss->facoli : 1;
           set_color(color, p->wtype);
           set_linewidth(gkss->bwidth);
-          if (gkss->clip_tnr != 0)
-            {
-              packb("gsave");
-              set_clip(gkss->viewport[gkss->clip_tnr]);
-            }
+          packb("gsave");
+          set_clip_rect(gkss->cntnr);
           if (style == GKS_K_INTSTYLE_SOLID)
             fill_routine(ia[0], r1, r2, tnr);
           else if (style == GKS_K_INTSTYLE_PATTERN)
@@ -2118,10 +2128,7 @@ void gks_drv_ps(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r
               yres = 1.0 / 4650.0;
               gks_emul_fillarea(ia[0], r1, r2, tnr, line_routine, yres);
             }
-          if (gkss->clip_tnr != 0)
-            {
-              packb("grestore");
-            }
+          packb("grestore");
           p->empty = 0;
         }
       break;
@@ -2139,16 +2146,10 @@ void gks_drv_ps(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r
               p->init = 1;
             }
           gks_set_dev_xform(gkss, p->window, p->viewpt);
-          if (gkss->clip_tnr != 0)
-            {
-              packb("gsave");
-              set_clip(gkss->viewport[gkss->clip_tnr]);
-            }
+          packb("gsave");
+          set_clip_rect(gkss->cntnr);
           cell_array(r1[0], r1[1], r2[0], r2[1], dx, dy, dimx, ia, p->wtype, true_color);
-          if (gkss->clip_tnr != 0)
-            {
-              packb("grestore");
-            }
+          packb("grestore");
           p->empty = 0;
         }
       break;
@@ -2163,16 +2164,10 @@ void gks_drv_ps(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r
               p->init = 1;
             }
           gks_set_dev_xform(gkss, p->window, p->viewpt);
-          if (gkss->clip_tnr != 0)
-            {
-              packb("gsave");
-              set_clip(gkss->viewport[gkss->clip_tnr]);
-            }
+          packb("gsave");
+          set_clip_rect(gkss->cntnr);
           gdp(ia[0], r1, r2, ia[1], ia[2], ia + 3);
-          if (gkss->clip_tnr != 0)
-            {
-              packb("grestore");
-            }
+          packb("grestore");
           p->empty = 0;
         }
       break;
@@ -2200,7 +2195,6 @@ void gks_drv_ps(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r
       p->window[3] = r2[1];
       set_xform(p->window, p->viewpt);
       init_norm_xform();
-      if (p->init) set_clip(p->window);
       break;
 
       /* set workstation viewport */
@@ -2213,6 +2207,12 @@ void gks_drv_ps(int fctid, int dx, int dy, int dimx, int *ia, int lr1, double *r
       init_norm_xform();
       break;
 
+      /* set nominal size */
+    case 109:
+      p->nominal_size = MIN(p->width, p->height) / 500.0 * 72 / 600;
+      if (gkss->nominal_size > 0) p->nominal_size *= gkss->nominal_size;
+      break;
+
     default:;
     }
 }
