@@ -51,6 +51,77 @@ typedef enum
 
 #define GR_DEFAULT_MATH_FONT 232
 
+#define GR_COLORMAP_DEFAULT 0
+#define GR_COLORMAP_TEMPERATURE 1
+#define GR_COLORMAP_GRAYSCALE 2
+#define GR_COLORMAP_GLOWING 3
+#define GR_COLORMAP_RAINBOWLIKE 4
+#define GR_COLORMAP_GEOLOGIC 5
+#define GR_COLORMAP_GREENSCALE 6
+#define GR_COLORMAP_CYANSCALE 7
+#define GR_COLORMAP_BLUESCALE 8
+#define GR_COLORMAP_MAGENTASCALE 9
+#define GR_COLORMAP_REDSCALE 10
+#define GR_COLORMAP_FLAME 11
+#define GR_COLORMAP_BROWNSCALE 12
+#define GR_COLORMAP_PILATUS 13
+#define GR_COLORMAP_AUTUMN 14
+#define GR_COLORMAP_BONE 15
+#define GR_COLORMAP_COOL 16
+#define GR_COLORMAP_COPPER 17
+#define GR_COLORMAP_GRAY 18
+#define GR_COLORMAP_HOT 19
+#define GR_COLORMAP_HSV 20
+#define GR_COLORMAP_JET 21
+#define GR_COLORMAP_PINK 22
+#define GR_COLORMAP_SPECTRAL 23
+#define GR_COLORMAP_SPRING 24
+#define GR_COLORMAP_SUMMER 25
+#define GR_COLORMAP_WINTER 26
+#define GR_COLORMAP_GIST_EARTH 27
+#define GR_COLORMAP_GIST_HEAT 28
+#define GR_COLORMAP_GIST_NCAR 29
+#define GR_COLORMAP_GIST_RAINBOW 30
+#define GR_COLORMAP_GIST_STERN 31
+#define GR_COLORMAP_AFMHOT 32
+#define GR_COLORMAP_BRG 33
+#define GR_COLORMAP_BWR 34
+#define GR_COLORMAP_COOLWARM 35
+#define GR_COLORMAP_CMRMAP 36
+#define GR_COLORMAP_CUBEHELIX 37
+#define GR_COLORMAP_GNUPLOT 38
+#define GR_COLORMAP_GNUPLOT2 39
+#define GR_COLORMAP_OCEAN 40
+#define GR_COLORMAP_RAINBOW 41
+#define GR_COLORMAP_SEISMIC 42
+#define GR_COLORMAP_TERRAIN 43
+#define GR_COLORMAP_VIRIDIS 44
+#define GR_COLORMAP_INFERNO 45
+#define GR_COLORMAP_PLASMA 46
+#define GR_COLORMAP_MAGMA 47
+#define GR_COLORMAP_UNIFORM 48
+
+#define GR_COLOR_RED 236
+#define GR_COLOR_GREEN 237
+#define GR_COLOR_YELLOW 238
+#define GR_COLOR_BLUE 239
+#define GR_COLOR_ORANGE 240
+#define GR_COLOR_PURPLE 241
+#define GR_COLOR_CYAN 242
+#define GR_COLOR_MAGENTA 243
+#define GR_COLOR_LIME 244
+#define GR_COLOR_PINK 245
+#define GR_COLOR_TEAL 246
+#define GR_COLOR_LAVENDER 247
+#define GR_COLOR_BROWN 248
+#define GR_COLOR_BEIGE 249
+#define GR_COLOR_MAROON 250
+#define GR_COLOR_MINT 251
+#define GR_COLOR_OLIVE 252
+#define GR_COLOR_APRICOT 253
+#define GR_COLOR_NAVY 254
+#define GR_COLOR_GREY 255
+
 typedef enum
 {
   GR_OPTION_X_LOG = 1u << 0u,
@@ -67,14 +138,14 @@ typedef enum
   GR_OPTION_Z_LOG2 = 1u << 8u,
   GR_OPTION_X_LN = 1u << 9u,
   GR_OPTION_Y_LN = 1u << 10u,
-  GR_OPTION_Z_LN = 1u << 11u,
+  GR_OPTION_Z_LN = 1u << 11u
 } scale_option_t;
 
 typedef enum
 {
   GR_SPEC_LINE = 1u << 0u,
   GR_SPEC_MARKER = 1u << 1u,
-  GR_SPEC_COLOR = 1u << 2u,
+  GR_SPEC_COLOR = 1u << 2u
 } linespec_t;
 
 typedef enum
@@ -86,7 +157,7 @@ typedef enum
   GR_OPTION_COLORED_MESH,
   GR_OPTION_CELL_ARRAY,
   GR_OPTION_SHADED_MESH,
-  GR_OPTION_3D_MESH, /* for GR3 */
+  GR_OPTION_3D_MESH /* for GR3 */
 } surface_option_t;
 
 typedef enum
@@ -97,24 +168,32 @@ typedef enum
 
 typedef enum
 {
-  GR_STOP = 0,
-  GR_MOVETO = 1,
-  GR_LINETO = 2,
-  GR_CURVE3 = 3,
-  GR_CURVE4 = 4,
-  GR_CLOSEPOLY = 0x4f,
+  GR_STOP,
+  GR_MOVETO,
+  GR_LINETO,
+  GR_CURVE3,
+  GR_CURVE4,
+  GR_CLOSEPOLY = 0x4f
 } path_code_t;
 
 typedef enum
 {
-  GR_XFORM_BOOLEAN = 0,
-  GR_XFORM_LINEAR = 1,
-  GR_XFORM_LOG = 2,
-  GR_XFORM_LOGLOG = 3,
-  GR_XFORM_CUBIC = 4,
-  GR_XFORM_EQUALIZED = 5,
+  GR_XFORM_BOOLEAN,
+  GR_XFORM_LINEAR,
+  GR_XFORM_LOG,
+  GR_XFORM_LOGLOG,
+  GR_XFORM_CUBIC,
+  GR_XFORM_EQUALIZED
 } xform_types_t;
 
+typedef enum
+{
+  GR_INTERP2_NEAREST,
+  GR_INTERP2_LINEAR,
+  GR_INTERP2_SPLINE,
+  GR_INTERP2_CUBIC
+} interp2_method_t;
+
 typedef struct
 {
   double x, y;
@@ -172,6 +251,39 @@ typedef struct
   int decimal_digits;
 } format_reference_t;
 
+typedef struct
+{
+  double value;
+  int is_major;
+} tick_t;
+
+typedef struct
+{
+  double tick;
+  char *label;
+  double width;
+} tick_label_t;
+
+typedef struct
+{
+  double min, max;
+  double tick, org;
+  double position;
+  int major_count;
+  int num_ticks;
+  tick_t *ticks;
+  double tick_size;
+  int num_tick_labels;
+  tick_label_t *tick_labels;
+  double label_position;
+  int draw_axis_line;
+  int label_orientation;
+} axis_t;
+
+#define GR_AXES_SIMPLE_AXES (1 << 0)
+#define GR_AXES_TWIN_AXES (1 << 1)
+#define GR_AXES_WITH_GRID (1 << 2)
+
 DLLEXPORT void gr_initgr(void);
 DLLEXPORT int gr_debug(void);
 DLLEXPORT void gr_opengks(void);
@@ -227,8 +339,8 @@ DLLEXPORT void gr_setfillstyle(int);
 DLLEXPORT void gr_inqfillstyle(int *);
 DLLEXPORT void gr_setfillcolorind(int);
 DLLEXPORT void gr_inqfillcolorind(int *);
-DLLEXPORT void gr_setresizebehaviour(int);
-DLLEXPORT void gr_inqresizebehaviour(int *);
+DLLEXPORT void gr_setnominalsize(double);
+DLLEXPORT void gr_inqnominalsize(double *);
 DLLEXPORT void gr_setcolorrep(int, double, double, double);
 DLLEXPORT void gr_setwindow(double, double, double, double);
 DLLEXPORT void gr_inqwindow(double *, double *, double *, double *);
@@ -257,6 +369,10 @@ DLLEXPORT void gr_axes(double, double, double, double, int, int, double);
 DLLEXPORT void gr_axeslbl(double, double, double, double, int, int, double,
                           void (*)(double, double, const char *, double),
                           void (*)(double, double, const char *, double));
+DLLEXPORT void gr_axis(char, axis_t *);
+DLLEXPORT void gr_drawaxis(char, axis_t *);
+DLLEXPORT void gr_drawaxes(axis_t *, axis_t *, int);
+DLLEXPORT void gr_freeaxis(axis_t *);
 DLLEXPORT void gr_grid(double, double, double, double, int, int);
 DLLEXPORT void gr_grid3d(double, double, double, double, double, double, int, int, int);
 DLLEXPORT void gr_verrorbars(int, double *, double *, double *, double *);
@@ -310,6 +426,8 @@ DLLEXPORT void gr_endgraphics(void);
 DLLEXPORT char *gr_getgraphics(void);
 DLLEXPORT int gr_drawgraphics(char *);
 DLLEXPORT int gr_startlistener(void);
+DLLEXPORT int gr_inqgrplotport(void);
+DLLEXPORT int gr_setgrplotport(int);
 DLLEXPORT void gr_mathtex(double, double, char *);
 DLLEXPORT void gr_inqmathtex(double, double, char *, double *, double *);
 DLLEXPORT void gr_mathtex3d(double, double, double, char *, int);
@@ -321,6 +439,8 @@ DLLEXPORT void gr_cancelbboxcallback(void);
 DLLEXPORT void gr_moveselection(double, double);
 DLLEXPORT void gr_resizeselection(int, double, double);
 DLLEXPORT void gr_inqbbox(double *, double *, double *, double *);
+DLLEXPORT void gr_setbackground(void);
+DLLEXPORT void gr_clearbackground(void);
 DLLEXPORT double gr_precision(void);
 DLLEXPORT int gr_text_maxsize(void);
 DLLEXPORT void gr_setregenflags(int);
@@ -338,7 +458,7 @@ DLLEXPORT void gr_trisurface(int, double *, double *, double *);
 DLLEXPORT void gr_gradient(int, int, double *, double *, double *, double *, double *);
 DLLEXPORT void gr_quiver(int, int, double *, double *, double *, double *, int);
 DLLEXPORT void gr_interp2(int nx, int ny, const double *x, const double *y, const double *z, int nxq, int nyq,
-                          const double *xq, const double *yq, double *zq, int method, double extrapval);
+                          const double *xq, const double *yq, double *zq, interp2_method_t method, double extrapval);
 DLLEXPORT const char *gr_version(void);
 DLLEXPORT void gr_shade(int, double *, double *, int, int, double *, int, int, int *);
 DLLEXPORT void gr_shadepoints(int, double *, double *, int, int, int);
@@ -354,6 +474,7 @@ DLLEXPORT void gr_setbordercolorind(int);
 DLLEXPORT void gr_inqbordercolorind(int *);
 DLLEXPORT void gr_selectclipxform(int);
 DLLEXPORT void gr_inqclipxform(int *);
+DLLEXPORT void gr_inqclip(int *, double *);
 DLLEXPORT void gr_setprojectiontype(int);
 DLLEXPORT void gr_inqprojectiontype(int *);
 DLLEXPORT void gr_setperspectiveprojection(double, double, double);
@@ -402,6 +523,8 @@ DLLEXPORT void gr_setmathfont(int font);
 DLLEXPORT void gr_inqmathfont(int *font);
 DLLEXPORT void gr_setclipregion(int region);
 DLLEXPORT void gr_inqclipregion(int *region);
+DLLEXPORT void gr_setclipsector(double start_angle, double end_angle);
+DLLEXPORT void gr_inqclipsector(double *start_angle, double *end_angle);
 DLLEXPORT void gr_settextoffset(double xoff, double yoff);
 DLLEXPORT char *gr_ftoa(char *string, double value, format_reference_t *reference);
 DLLEXPORT void gr_getformat(format_reference_t *result, double origin, double min, double max, double tick_width,
